%Code to replicate the power simulation results of Andrews and Mikusheva
%(2014), "Maximum Likelihood Inference in Weakly Identified DSGE Models"

clear
global point_list loglike_list diff_precision  estim_params derivative_calc partial_list
global test_params calibrated_values AM_LM_output true_value Qu_LM_PW_A
global options fixJ Jsim Jmean Qu_LM_output AM_LM_J_fixed Qu_LM_prewhitened
global likelihood_eval point_test composite_test AM_LM Qu_LM Qu_LM_PW_output
global simpoint paramvec GQIKtheta data J_MC_integration J_MC_reps

%This code is designed to work with Dynare (version 4.1.1), avalible at:
%http://www.dynare.org/
%and also uses commands from the e4 timeseries package for Matlab,
%availible at:
%http://www.ucm.es/info/icae/e4/

powergrid=10;
powerstep=.8;
MCreps=1000;

diff_precision=10^-6;
derivative_calc=1;


%Options reported in paper:
%Core power simulations: powergrid=10, powerstep=.8, MCreps=500
%Size simulations: powergrid=0, MCreps=1000.  To test composite hypotheses,
%set composite_test=1, and set the elements of "test_params" 
%corresponding to parameters not under test to zero.

datalength=403;

Jreps=10000;

prewhiten_sim_length=5100;

%The choices availible are:
% 1: point_test==1: tests the simple null of the base parameter value.
% 2: composite_test==1: tests that the parameters indicated in test_params
%    are equal to the base parameter values.
%
% 3: availible tests: AM_LM==1 (Andrews Mikusheva LM test: using RMLE if
%                     testing composite hypothesis)
%                       fixJ==1: Also computes LM statistic using expected
%                       information
%                     Qu_LM==1 (Approximate frequency-domain LM test proposed
%                     by Zhongjun Qu)
%                       Qu_LM_prewhitened: also computes a version of Qu's
%                       test which prewhitens the data, which improves size
%                       control for persistent data.
%                     GQIK==1 (LR test proposed by Guerron-Quintana Inoue
%                     and Killian)
%                       restim==1: Start search for unresteicted optimum at
%                       the resticted MLE (i.e. MLE in the DSGE model,
%                       rather than at null parameter value).
%                     DKK==1 (Limited Information Statistic Proposed by
%                     Dufour Khalaf and Kichian)
%                       DKK_var_fix==1: Use average of long run variance
%                       estimates (using fact that MA(1) process),
%                       rather than recalculating in each simulation.
%                       DKK_lags==k: use k lags in calculation of long-run
%                       variance matrix by Newey-West
% 4: test_params: by setting a given parameter to zero in test_params, we
%    plug in the restricted MLE for that parameter when conducting composite tests.

likelihood_eval=0;
point_test=1;
composite_test=0;

%For composite tests, calculates information matrix in LM_{e} by
%monte-carlo integregration.  Seems too computationally costly relative to
%calculating information directly following Iskrev.
J_MC_integration=0;
J_MC_reps=1000;

AM_LM=1;
fixJ=1;
recalcJ=1;

Qu_LM=1;
Qu_LM_prewhitened=1;

GQIK=1;
restim=1;

DKK=1;
DKK_var_fix=0;
DKK_lags=3;


e4init
sete4opt('maxiter',200,'step',.2,'vcond','lyapunov');

%Diagonal of information matrix: used for scaling in power simulations
Infodiag=[29.3845538593333;...
    9.70936762182429;...
    25291.7874846555;...
    5398.18378923933;...
    444.932969543992;...
    1150.80274488274;...
    3795.00987064585;...
    5686.86902710448;...
    1293.46977708974];

estim_params=ones(1,9);
test_params=[...
    1.0... %phi_x
    1.0... %phi_pie
    1.0... %alpha
    1.0... %rho
    1.0... %delta
    1.0... %kappa
    1.0... %sigma_a
    1.0... %sigma_u
    1.0... %sigma
    ];


label_vec={...
    '\phi_x';... %phi_x
    '\phi_{\pi}';... %phi_pie
    '\lambda';... %alpha
    '\rho';... %rho
    '\delta';... %delta
    '\kappa';... %kappa
    '\sigma_a';... %sigma_a
    '\sigma_u';... %sigma_u
    '\sigma'... %sigma
    };

beta=.99;

%Modified MLE based on Smets and Wouters Data
true_value=[...
    2.2775... %phi_x
    2.0229... %phi_pie
    0.898... %alpha
    0.85... %rho
    0.1028... %delta
    0.1... %kappa
    0.3247... %sigma_a
    0.2649... %sigma_u
    0.5561... %sigma
    ];


Jsim=0;
loglike_list=NaN;
partial_list=NaN;
calibrated_values=true_value;
point_list=calibrated_values;

tic

%Calculate the Fisher information for use in the LM statistic
%Order of variables for code: rho*da r u da x pi (note redundancy of da)
if fixJ==1;
    if recalcJ==0
        load ExpectedJ
    else
        paramvec=true_value;
        Base_model_estimation_likeeval_modified
        Phi=[zeros(6,1) oo_.dr.ghx zeros(6,2)];
        %Phi=[oo_.dr.ghx(:,1) zeros(6,1) oo_.dr.ghx(:,2:3) zeros(6,2)];
        E=oo_.dr.ghu;
        
        H=[0 1 0 0 0 0;...
            0 0 0 0 1 0;...
            0 0 0 0 0 1];
        [theta, din, lab] = ss2thd(Phi, [], E, H, [], [], eye(3), [], []);
        
        [std, corrm, varm, Im] = imod(theta, din, zeros(200,3), 0);
        
        
        p.phix=true_value(1);
        p.phipi=true_value(2);
        p.alpha=true_value(3);
        p.rho=true_value(4);
        p.delta=true_value(5);
        p.kappa=true_value(6);
        p.sigmaa=true_value(7);
        p.sigmau=true_value(8);
        p.sigma=true_value(9);
        p.beta=.99;
        
        G0=[1 0 0 -p.rho 0 0;...
            0 -1 1 0 (1-p.alpha)*p.phix (1-p.alpha)*p.phipi;...
            0 0 1 0 0 0;...
            0 0 0 1 0 0;...
            1 -1 0 0 -1 0;...
            0 0 0 0 p.kappa -1];
        
        
        
        G1=[0 0 0 0 0 0;...
            0 0 0 0 0 0;...
            0 0 0 0 0 0;...
            0 0 0 0 0 0;...
            0 0 0 0 -1 -1;...
            0 0 0 0 0 -p.beta];
        
        G2=[0 0 0 0 0 0;...
            0 -p.alpha 0 0 0 0;...
            0 0 p.delta 0 0 0;...
            0 0 0 p.rho 0 0;...
            0 0 0 0 0 0;...
            0 0 0 0 0 0];
        
        G3=[0 0 0;...
            0 0 0;...
            p.sigmau 0 0;...
            0 p.sigmaa 0;...
            0 0 0;...
            0 0 p.sigma];
        
        dG1dt=zeros(size(G1(1:end)',1),9);
        dG2dt=zeros(size(G2(1:end)',1),9);
        dG3dt=zeros(size(G3(1:end)',1),9);
        dG0dt=zeros(size(G0(1:end)',1),9);
        
        %Set nonzero elements of dG0dt
        dG0dt(19,4)=-1;
        dG0dt(26,3)=-p.phix;
        dG0dt(26,1)=1-p.alpha;
        dG0dt(30,6)=1;
        dG0dt(32,3)=-p.phipi;
        dG0dt(32,2)=1-p.alpha;
        
        %All elements of dG1dt are zero
        
        %Set nonzero elements of dG2dt
        dG2dt(8,3)=-1;
        dG2dt(15,5)=1;
        dG2dt(22,4)=1;
        
        %Set nonzero elements of dG3dt
        dG3dt(3,8)=1;
        dG3dt(10,7)=1;
        dG3dt(18,9)=1;
        
        %Use Dynare to generate parameters of state-space representation
        paramvec=true_value;
        Base_model_estimation_likeeval_modified
        
        %Phi=[oo_.dr.ghx(:,1) zeros(6,1) oo_.dr.ghx(:,2:3) zeros(6,2)];
        %A=[oo_.dr.ghx(:,1) zeros(6,1) oo_.dr.ghx(:,2:3) zeros(6,2)];
        A=[zeros(6,1) oo_.dr.ghx zeros(6,2)];
        B=oo_.dr.ghu;
        
        dAdt=-(kron(eye(6),G0)-kron(A',G1)-kron(eye(6),G1*A))^-1*...
            (kron(A',eye(6))*dG0dt-kron((A')*(A'),eye(6))*dG1dt-dG2dt);
        
        dBdt=(kron(eye(3),(G0-G1*A)))^-1*...
            (dG3dt-kron(B',eye(6))*(dG0dt-kron(A',eye(6))*dG1dt-kron(eye(6),G1)*dAdt));
        
        
        dPhidt2=[dAdt; dBdt; zeros(24,9)];
        
        Jmean=dPhidt2'*Im*dPhidt2;
    end
end

%Calculates state space parameters under null
if GQIK==1
    paramvec=true_value;
    Base_model_estimation_likeeval_modified
    
    Phi0=[zeros(6,1) oo_.dr.ghx zeros(6,2)];
    E0=oo_.dr.ghu;
    
    H0=[0 1 0 0 0 0;...
        0 0 0 0 1 0;...
        0 0 0 0 0 1];
    LRV0=dlyap(Phi0,E0*E0');
end

%Calculates state space parameters under null and, if using average LRV
%estimate, computes average over Jreps simulations.
if DKK==1
    %paramvec=true_value;
    %Base_model_estimation_likeeval_modified
    
    simpoint=true_value;
    Base_model_power_mod2
    
    Phi0=[zeros(6,1) oo_.dr.ghx zeros(6,2)];
    E0=oo_.dr.ghu;
    H0=[0 1 0 0 0 0;...
        0 0 0 0 1 0;...
        0 0 0 0 0 1];
    
    LRV0=dlyap(Phi0,E0*E0');
    if DKK_var_fix==1
        Sigma2store=NaN(9,9,Jreps);
        for n=1:Jreps
            e=randn([3,datalength]);
            S=NaN(6,datalength);
            S(:,1)=real(LRV0^.5)*randn(6,1);
            for k=2:datalength;
                S(:,k)=Phi0*S(:,k-1)+E0*e(:,k);
            end
            obs=H0*S;
            
            r=obs(1,:)';
            x=obs(2,:)';
            pie=obs(3,:)';
            xcut=x(101:300,:);
            piecut=pie(101:300,:);
            rcut=r(101:300,:);
            
            calibrated_value=true_value;
            
            p.phix=calibrated_value(1);
            p.phipi=calibrated_value(2);
            p.alpha=calibrated_value(3);
            p.rho=calibrated_value(4);
            p.delta=calibrated_value(5);
            p.kappa=calibrated_value(6);
            p.sigmaa=calibrated_value(7);
            p.sigmau=calibrated_value(8);
            p.sigma=calibrated_value(9);
            p.beta=.99;
            
            et=piecut(2:end-1,:)-p.beta*piecut(3:end,:)-p.kappa*xcut(2:end-1,:);
            xi_pi=et(2:end,:);
            
            rrt=xcut(2:end-1,:)+rcut(2:end-1,:)-piecut(3:end,:)-xcut(3:end,:);
            xi_x=rrt(2:end,:)-p.rho*rrt(1:end-1,:);
            
            ut=rcut(2:end-1,:)-p.alpha*rcut(1:end-2,:)-(1-p.alpha)*p.phipi*piecut(2:end-1,:)-(1-p.alpha)*p.phix*xcut(2:end-1,:);
            xi_r=ut(2:end,:)-p.rho*ut(1:end-1,:);
            
            xi=[xi_pi xi_x xi_r];
            Z=[xcut(1:end-3,:) rcut(1:end-3,:) piecut(1:end-3,:)];
            
            
            b1=regress(xi_pi,Z);
            b2=regress(xi_x,Z);
            b3=regress(xi_r,Z);
            
            b=[b1; b2; b3];
            
            v1=xi_pi-Z*b1;
            v2=xi_x-Z*b2;
            v3=xi_r-Z*b3;
            
            v=[repmat(v1,1,3).*Z repmat(v2,1,3).*Z repmat(v3,1,3).*Z];
            T=size(Z,1);
            
            G2=1/T*(v')*v;
            G2(:,:,2)=1/T*((v(2:end,:)'*v(1:end-1,:))+(v(2:end,:)'*v(1:end-1,:))');
            Sigma2=sum(G2,3);
            
            Sigma2store(:,:,n)=Sigma2;
        end
        Sigma2mean=mean(Sigma2store,3);
    end
    
end

if Qu_LM_prewhitened==1
    simpoint=true_value;
    Base_model_power_mod2
    
    Phi0=[zeros(6,1) oo_.dr.ghx zeros(6,2)];
    E0=oo_.dr.ghu;
    H0=[0 1 0 0 0 0;...
        0 0 0 0 1 0;...
        0 0 0 0 0 1];
    LRV0=dlyap(Phi0,E0*E0');
    e=randn([3,prewhiten_sim_length]);
    S=NaN(6,prewhiten_sim_length);
    S(:,1)=real(LRV0^.5)*randn(6,1);
    for k=2:prewhiten_sim_length;
        S(:,k)=Phi0*S(:,k-1)+E0*e(:,k);
    end
    obs=H0*S;
    
    r=obs(1,:)';
    x=obs(2,:)';
    pie=obs(3,:)';
    xcut=x(101:end,:);
    piecut=pie(101:end,:);
    rcut=r(101:end,:);
    
    %data_sim=[xcut piecut rcut];
    data_MC=[rcut xcut piecut];
     
    Qu_LM_PW_A=(data_MC(1:end-1,:)'*data_MC(1:end-1,:))^-1*data_MC(1:end-1,:)'*data_MC(2:end,:);
    Qu_LM_PW_A=Qu_LM_PW_A';
end


AM_LM_output=NaN;
Qu_LM_output=NaN;
Qu_LM_PW_output=NaN;
AM_LM_J_fixed=NaN;

%Simulate test statistics
if point_test==1
    
    AM_LM_store=NaN(9,2*powergrid+1,MCreps);
    Qu_LM_store=NaN(9,2*powergrid+1,MCreps);
    Qu_LM_PW_store=NaN(9,2*powergrid+1,MCreps);
    AM_LM_fixedJ_store=NaN(9,2*powergrid+1,MCreps);
    DKK_store=NaN(9,2*powergrid+1,MCreps);
    DKK_fixed_store=NaN(9,2*powergrid+1,MCreps);
    GQIK_store=NaN(9,2*powergrid+1,MCreps);
    Const_check=zeros(9,2*powergrid+1);
    
    Phistore=NaN(6,6,9,2*powergrid+1);
    Estore=NaN(6,3,9,2*powergrid+1);
    
    simpoint=true_value;
    Base_model_power_mod2
    xcheck=x;
    
    %simulate data to check which points in powergrid are valid points
    %for simulation and to collects state-space parameter values
    for n=1:1
        for k=-powergrid:powergrid
            for m=1:9
                simpoint=true_value;
                simpoint(m)=simpoint(m)+k*powerstep/Infodiag(m)^.5;
                if k==0 &&m>1
                    
                else
                    Base_model_power_mod2
                    
                    if mean(x==xcheck)<1 && rank([x r pie])==3
                        Const_check(m,powergrid+k+1)=1;
                        
                        Phi=[zeros(6,1) oo_.dr.ghx zeros(6,2)];
                        E=oo_.dr.ghu;
                        
                        Phistore(:,:,m,powergrid+k+1)=Phi;
                        Estore(:,:,m,powergrid+k+1)=E;
                        H=[0 1 0 0 0 0;...
                            0 0 0 0 1 0;...
                            0 0 0 0 0 1];
                        
                        LRV=dlyap(Phi,E*E');
                        if min(eig(H*LRV*H'))<=0
                            Const_check(m,powergrid+k+1)=0;
                        end
                        
                    end
                    xcheck=x;
                end
                
            end
        end
    end
    
    %Simulate test statistics
    for n=1:MCreps
        for k=-powergrid:powergrid
            for m=1:9
                simpoint=true_value;
                simpoint(m)=simpoint(m)+k*powerstep/Infodiag(m)^.5;
                
                %simulate under null parameter value only once per "n"
                if k==0 &&m>1
                    if AM_LM==1;
                        AM_LM_store(m,k+powergrid+1,n)=AM_LM_store(1,k+powergrid+1,n);
                    end
                    
                    if Qu_LM==1;
                        Qu_LM_store(m,k+powergrid+1,n)= Qu_LM_store(1,k+powergrid+1,n);
                        if Qu_LM_prewhitened==1
                            Qu_LM_PW_store(m,k+powergrid+1,n)= Qu_LM_PW_store(1,k+powergrid+1,n);
                        end
                    end
                    
                    if fixJ==1;
                        AM_LM_fixedJ_store(m,k+powergrid+1,n)=AM_LM_fixedJ_store(1,k+powergrid+1,n);
                    end
                    
                    if GQIK==1;
                        GQIK_store(m,k+powergrid+1,n)= GQIK_store(1,k+powergrid+1,n);
                    end
                    
                    if DKK==1;
                        DKK_store(m,k+powergrid+1,n)=DKK_store(1,k+powergrid+1,n);
                    end
                    
                    if DKK==1;
                        DKK_fixed_store(m,k+powergrid+1,n)=DKK_fixed_store(1,k+powergrid+1,n);
                    end
                else
                    
                    n
                    k
                    m
                    %check that valid point for simulations
                    if Const_check(m,k+powergrid+1)==1;
                        %set seed for simulations, then simulate data using
                        %stored values of state-space parameters
                        randn('seed',n);
                        Phi=Phistore(:,:,m,powergrid+k+1);
                        E=Estore(:,:,m,powergrid+k+1);
                        H=[0 1 0 0 0 0;...
                            0 0 0 0 1 0;...
                            0 0 0 0 0 1];
                        
                        LRV=dlyap(Phi,E*E');
                        e=randn([3,datalength]);
                        S=NaN(6,datalength);
                        S(:,1)=real(LRV^.5)*randn(6,1);
                        for k1=2:datalength;
                            S(:,k1)=Phi*S(:,k1-1)+E*e(:,k1);
                        end
                        obs=H*S;
                        
                        r=obs(1,:)';
                        x=obs(2,:)';
                        pie=obs(3,:)';
                        
                        save data_sim r x pie
                        
                        %Use Dynare to evaluate AM_LM and Qu_LM
                        Base_model_estimation_testeval_fix
                        
                        
                        if AM_LM==1;
                            AM_LM_store(m,k+powergrid+1,n)=AM_LM_output;
                        end
                        
                        if Qu_LM==1;
                            Qu_LM_store(m,k+powergrid+1,n)=real(Qu_LM_output);
                            if Qu_LM_prewhitened==1
                                Qu_LM_PW_store(m,k+powergrid+1,n)=real(Qu_LM_PW_output);
                            end
                        end
                        
                        if fixJ==1;
                            AM_LM_fixedJ_store(m,k+powergrid+1,n)=AM_LM_J_fixed;
                        end
                        
                        %Evaluate GQIK LR statistic
                        if GQIK==1
                            
                            
                            [theta, din, lab] = ss2thd(Phi0, [], E0, H0, [], [], eye(3), [], []);
                            xcut=x(101:300,:);
                            piecut=pie(101:300,:);
                            rcut=r(101:300,:);
                            data=[rcut xcut piecut];
                            rlik=kalman_filter_adapted(Phi0,E0,eye(3),0,LRV0,data',1,[2 5 6]',10^-4,10^-4);
                            
                            %If restim==1, calculates the restricted MLE
                            %and begins unrestricted state space model
                            %estimation from there
                            if restim==1
                                Base_model_estimation_fix;
                                close all
                                Phi=[zeros(6,1) oo_.dr.ghx zeros(6,2)];
                                E=oo_.dr.ghu;
                                H=[0 1 0 0 0 0;...
                                    0 0 0 0 1 0;...
                                    0 0 0 0 0 1];
                                LRV=dlyap(Phi,E*E');
                                
                                
                                [theta, din, lab] = ss2thd(Phi, [], E, H, [], [], eye(3), [], []);
                                ulik2=kalman_filter_adapted(Phi,E,eye(3),0,LRV,data',1,[2 5 6]',10^-4,10^-4);
                                %Restricted LR statistic (restricted to
                                %DSGE model)
                                GIK_store_r(m,k+powergrid+1,n)=2*(rlik-ulik2);
                            end
                            
                            theta=[theta zeros(size(theta,1),1)];
                            theta(end,2)=1;
                            theta(end-2,2)=1;
                            theta(end-5,2)=1;
                            theta(58,2)=1;
                            theta(68,2)=1;
                            theta(72,2)=1;
                            
                            theta(abs(theta(:,1))<10^-6,1)=0;
                            theta(abs(theta(:,1))<10^-6,2)=1;
                            
                            %Here, fix parameters to make information for
                            %remaining parameters full rank to do this,
                            %drop elements with largest weight in
                            %eigenvectors corresponding to five smallest
                            %eignvalues of information under null.
                            theta(19,2)=1;
                            theta(23,2)=1;
                            theta(39,2)=1;
                            theta(43,2)=1;
                            
                            GQIKtheta=theta;
                            
                            options=optimset('MaxIter',20000,'MaxFunEvals',20000);
                            tic
                            x0=GQIKtheta(GQIKtheta(:,2)==0,1);
                            g=@RFlikeeval2;
                            [xhat ulik]=fminsearch(g,x0,options);
                            
                            toc
                            
                            GQIK_store(m,k+powergrid+1,n)=2*(rlik-ulik);
                        end
                        
                        %Calculate LI statistic proposed by Dufour Khalaf
                        %and Kichian
                        if DKK==1
                            xcut=x(101:300,:);
                            piecut=pie(101:300,:);
                            rcut=r(101:300,:);
                            
                            calibrated_value=true_value;
                            
                            p.phix=calibrated_value(1);
                            p.phipi=calibrated_value(2);
                            p.alpha=calibrated_value(3);
                            p.rho=calibrated_value(4);
                            p.delta=calibrated_value(5);
                            p.kappa=calibrated_value(6);
                            p.sigmaa=calibrated_value(7);
                            p.sigmau=calibrated_value(8);
                            p.sigma=calibrated_value(9);
                            p.beta=.99;
                            
                            et=piecut(2:end-1,:)-p.beta*piecut(3:end,:)-p.kappa*xcut(2:end-1,:);
                            xi_pi=et(2:end,:);
                            
                            rrt=xcut(2:end-1,:)+rcut(2:end-1,:)-piecut(3:end,:)-xcut(3:end,:);
                            xi_x=rrt(2:end,:)-p.rho*rrt(1:end-1,:);
                            
                            ut=rcut(2:end-1,:)-p.alpha*rcut(1:end-2,:)-(1-p.alpha)*p.phipi*piecut(2:end-1,:)-(1-p.alpha)*p.phix*xcut(2:end-1,:);
                            xi_r=ut(2:end,:)-p.rho*ut(1:end-1,:);
                            
                            xi=[xi_pi xi_x xi_r];
                            Z=[xcut(1:end-3,:) rcut(1:end-3,:) piecut(1:end-3,:)];
                            
                            
                            b1=regress(xi_pi,Z);
                            b2=regress(xi_x,Z);
                            b3=regress(xi_r,Z);
                            
                            b=[b1; b2; b3];
                            
                            v1=xi_pi-Z*b1;
                            v2=xi_x-Z*b2;
                            v3=xi_r-Z*b3;
                            
                            v=[repmat(v1,1,3).*Z repmat(v2,1,3).*Z repmat(v3,1,3).*Z];
                            T=size(Z,1);
                            
                            %Calculate Newey-West Estimator of Long-run
                            %variance
                            G2=1/T*(v')*v;
                            for k1=1:DKK_lags
                                G2(:,:,k1+1)=(1+DKK_lags-k1)/(1+DKK_lags)*1/T*((v(1+k1:end,:)'*v(1:end-k1,:))+(v(1+k1:end,:)'*v(1:end-k1,:))');
                            end
                            Sigma2=sum(G2,3);
                            
                            V=kron(eye(3),(Z'*Z/T)^-1)*Sigma2*kron(eye(3),(Z'*Z/T)^-1);
                            DKK_store(m,k+powergrid+1,n)=T*b'*V^-1*b;
                            bstore2(:,n)=b;
                            %Calculate statistic plugging in expected value
                            %of long run variance
                            if DKK_var_fix==1
                                
                                V=kron(eye(3),(Z'*Z/T)^-1)*Sigma2mean*kron(eye(3),(Z'*Z/T)^-1);
                                DKK_fixed_store(m,k+powergrid+1,n)=T*b'*V^-1*b;
                            end
                            
                        end
                        
                    end
                end
                
            end
        end
        if floor(n/3)==n/3
            save test_comparison_backup_alt
        end
    end
    cutoff=chi2inv(.95,9);
    GQIK_cutoff=chi2inv(.95,20);
    
    AM_LM_power=mean(AM_LM_store>cutoff,3);
    AM_LM_fixedJ_power=mean(AM_LM_fixedJ_store>cutoff,3);
    Qu_LM_power=mean(Qu_LM_store>cutoff,3);
    Qu_LM_PW_power=mean(Qu_LM_PW_store>cutoff,3);
    GQIK_power=mean(GQIK_store>GQIK_cutoff,3);
    DKK_power=mean(DKK_store>cutoff,3);
    DKK_fixed_power=mean(DKK_fixed_store>cutoff,3);
    
    AM_LM_power(isnan(sum(AM_LM_store,3)))=NaN;
    AM_LM_fixedJ_power(isnan(sum(AM_LM_fixedJ_store,3)))=NaN;
    Qu_LM_power(isnan(sum(Qu_LM_store,3)))=NaN;
    GQIK_power(isnan(sum(GQIK_store,3)))=NaN;
    DKK_power(isnan(sum(DKK_store,3)))=NaN;
    DKK_fixed_power(isnan(sum(DKK_store,3)))=NaN;
    
    close all
    for k=1:9
        figure(k)
        hold on
        x=true_value(k)-powergrid*powerstep/Infodiag(k)^.5:powerstep/Infodiag(k)^.5:true_value(k)+powergrid*powerstep/Infodiag(k)^.5;
        plot(x(1,isnan(AM_LM_power(k,:))==0),AM_LM_power(k,isnan(AM_LM_power(k,:))==0),'-s','LineWidth',2);
        plot(x(1,isnan(AM_LM_fixedJ_power(k,:))==0),AM_LM_fixedJ_power(k,isnan(AM_LM_fixedJ_power(k,:))==0),'-ko','LineWidth',2);
        plot(x(1,isnan(Qu_LM_power(k,:))==0),Qu_LM_power(k,isnan(Qu_LM_power(k,:))==0),'-r+','LineWidth',2);
        plot(x(1,isnan(Qu_LM_PW_power(k,:))==0),Qu_LM_PW_power(k,isnan(Qu_LM_PW_power(k,:))==0),'-cv','LineWidth',2);
        plot(x(1,isnan(DKK_power(k,:))==0),DKK_power(k,isnan(DKK_power(k,:))==0),'-g^','LineWidth',2);
        legend('AM LM','AM LM EInfo','Qu LM','Qu LM Prewhitened','DKK LI','Location','BestOutside')
        title(label_vec{k,:});
        hold off
    end
elseif composite_test==1
    %Calculates LM test for composite hypotheses
    AM_LM_store=NaN(sum(test_params),2*powergrid+1,MCreps);
    AM_LM_fixedJ_store=NaN(sum(test_params),2*powergrid+1,MCreps);
    Const_check=zeros(sum(test_params),2*powergrid+1);
    
    Phistore=NaN(6,6,sum(test_params),2*powergrid+1);
    Estore=NaN(6,3,sum(test_params),2*powergrid+1);
    
    a=1:9;
    testtrack=a(test_params==1);
    
    simpoint=true_value;
    Base_model_power_mod2
    xcheck=x;
    %simulate data to check which points in powergrid are valid points
    %for simulation and to collects state-space parameter values
    for n=1:1
        for k=-powergrid:powergrid
            for m=1:sum(test_params)
                simpoint=true_value;
                simpoint(testtrack(m))=simpoint(testtrack(m))+k*powerstep/Infodiag(testtrack(m))^.5;
                if k==0 &&m>1
                    
                else
                    Base_model_power_mod2
                    
                    if mean(x==xcheck)<1 && rank([x r pie])==3
                        Const_check(m,powergrid+k+1)=1;
                        
                        Phi=[zeros(6,1) oo_.dr.ghx zeros(6,2)];
                        E=oo_.dr.ghu;
                        
                        Phistore(:,:,m,powergrid+k+1)=Phi;
                        Estore(:,:,m,powergrid+k+1)=E;
                        H=[0 1 0 0 0 0;...
                            0 0 0 0 1 0;...
                            0 0 0 0 0 1];
                        
                        LRV=dlyap(Phi,E*E');
                        if min(eig(H*LRV*H'))<=0
                            Const_check(m,powergrid+k+1)=0;
                        end
                        
                    end
                    xcheck=x;
                end
                
            end
        end
    end
    
    %Simulate test statistics
    for n=1:MCreps
        for k=-powergrid:powergrid
            for m=1:sum(test_params)
                simpoint=true_value;
                simpoint(testtrack(m))=simpoint(testtrack(m))+k*powerstep/Infodiag(testtrack(m))^.5;
                
                %simulate under null parameter value only once per "n"
                if k==0 &&m>1
                    if AM_LM==1;
                        AM_LM_store(m,k+powergrid+1,n)=AM_LM_store(1,k+powergrid+1,n);
                    end
                    
                    if fixJ==1;
                        AM_LM_fixedJ_store(m,k+powergrid+1,n)=AM_LM_fixedJ_store(1,k+powergrid+1,n);
                    end
                else
                    
                    n
                    k
                    m
                    
                    if Const_check(m,k+powergrid+1)==1;
                        randn('seed',n);
                        Phi=Phistore(:,:,m,powergrid+k+1);
                        E=Estore(:,:,m,powergrid+k+1);
                        H=[0 1 0 0 0 0;...
                            0 0 0 0 1 0;...
                            0 0 0 0 0 1];
                        
                        LRV=dlyap(Phi,E*E');
                        e=randn([3,datalength]);
                        S=NaN(6,datalength);
                        S(:,1)=real(LRV^.5)*randn(6,1);
                        for k1=2:datalength;
                            S(:,k1)=Phi*S(:,k1-1)+E*e(:,k1);
                        end
                        obs=H*S;
                        
                        r=obs(1,:)';
                        x=obs(2,:)';
                        pie=obs(3,:)';
                        
                        save data_sim r x pie
                        
                        Base_model_estimation_testeval_fix
                        
                        if AM_LM==1;
                            AM_LM_store(m,k+powergrid+1,n)=AM_LM_output;
                        end
                        
                        if fixJ==1;
                            AM_LM_fixedJ_store(m,k+powergrid+1,n)=AM_LM_J_fixed;
                        end
                        
                    end
                end
                
            end
        end
        if floor(n/3)==n/3
            save test_comparison_backup_alt2
        end
    end
    cutoff=chi2inv(.95,sum(test_params));
    AM_LM_power=mean(AM_LM_store>cutoff,3);
    AM_LM_fixedJ_power=mean(AM_LM_fixedJ_store>cutoff,3);
    
    AM_LM_power(isnan(sum(AM_LM_store,3)))=NaN;
    AM_LM_fixedJ_power(isnan(sum(AM_LM_fixedJ_store,3)))=NaN;
end

toc